# -*- coding: utf-8 -*-
#
# Copyright (c) 2020~2999 - Cologler <skyoflw@gmail.com>
# ----------
#
# ----------

from collections.abc import Generator, Hashable
from contextlib import nullcontext
from logging import getLogger
from threading import Lock
from typing import Any

from ._internal import Disposable

_NULL_CONTEXT = nullcontext()
_logger = getLogger(__name__)

class ServicesMap[TK: Hashable, TV]:
    def __init__(self, *maps: dict[TK, list[tuple[TV]]], use_lock: bool=True) -> None:
        self._lock = Lock() if use_lock else _NULL_CONTEXT
        self._frozen_keys = set()
        self.maps: list[dict[TK, list[tuple[TV]]]] = list(maps) or [{}]

    def resolve(self, key: TK) -> Generator[TV, Any, None]:
        '''
        Resolve values with reversed order.
        '''
        with self._lock:
            for mapping in self.maps:
                yield from (t[0] for t in reversed(mapping.get(key, ())))

    def add(self, key: TK, value: TV) -> Disposable:

        with self._lock:
            if key in self._frozen_keys:
                raise RuntimeError(f'Key {key!r} is frozen.')

            record = (value,) # ensure dispose the right value
            vec = self.maps[0].setdefault(key, [])
            inserted_index = len(vec) # no insert func, so we can cache this index
            vec.append(record)

        def dispose() -> None:
            try:
                with self._lock:
                    # in most case, the del is del from end.
                    for i in range(min(inserted_index, len(vec) - 1), -1, -1):
                        if vec[i] is record:
                            del vec[i]
            except ValueError:
                _logger.warning('dispose() is called after the key be removed.')

        return Disposable(dispose)

    def freeze_key(self, key: TK) -> None:
        with self._lock:
            self._frozen_keys.add(key)

    def __setitem__(self, key: TK, value: TV) -> None:
        self.add(key, value)

    def __getitem__(self, key: TK) -> TV:
        'get item or raise `KeyError`` if not found'
        for value in self.resolve(key):
            return value
        raise KeyError(key)

    def get[TD](self, key: TK, default: TD=None) -> TV | TD:
        'get item or `default` if not found'
        for value in self.resolve(key):
            return value
        return default

    def get_many(self, key: TK) -> list[TV]:
        'get items as list'
        return list(self.resolve(key))

    def scope(self, use_lock: bool=False) -> 'ServicesMap':
        return self.__class__({}, *self.maps, use_lock=use_lock)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018~2999 - Cologler <skyoflw@gmail.com>
# ----------
#
# ----------

import inspect
import types
from collections.abc import Hashable, Mapping
from contextlib import ExitStack, nullcontext
from logging import getLogger
from threading import RLock
from types import MappingProxyType
from typing import Any, Callable, Iterable, Optional, Self, Type, overload, override

from ._bases import AllSupportsContext, IServiceProvider, LifeTime
from ._consts import SERVICEPROVIDER_NAMING_CONVENTION
from ._internal import Disposable, LockedMapping
from ._service_info import (
    BindedServiceInfo,
    GetAttrServiceInfo,
    GetGroupServiceInfo,
    IServiceInfo,
    ProviderServiceInfo,
    ValueServiceInfo,
)
from ._service_info.extra import CallerFrameServiceInfo, create_lifetime_service_info
from ._servicesmap import ServicesMap
from ._utils import wrap_signature as wrap_signature
from .err import ServiceNotFoundError
from .ioc_resolver import ServiceInfoChainResolver
from .keys import NamedType, _NamedTypeListKey
from .symbols import Symbols, TypedSymbol

_NULL_CONTEXT = nullcontext()

_logger = getLogger(__name__)


class ServiceProvider(IServiceProvider):
    def __init__(self,
            auto_enter: bool=False, *,
            # internal uses:
            _services: Optional[ServicesMap[Hashable, IServiceInfo]]=None,
            _parent: Optional['ServiceProvider']=None,
            _use_lock: bool=True,
        ) -> None:

        self._exit_stack = None
        self._scoped_cache = LockedMapping(use_lock=_use_lock)
        self._lock = RLock() if _use_lock else _NULL_CONTEXT
        self._parent = _parent

        assert (_parent is None) is (_services is None)

        if _parent is not None:
            assert _services is not None
            # scope provider
            assert auto_enter is False, 'must be default value'
            self._services = _services
            self._root: ServiceProvider = _parent._root

        else:
            assert _use_lock, 'root provider must use lock'
            # root provider
            self._services = ServicesMap(use_lock=True)
            self._root: ServiceProvider = self

            # serviceinfos
            get_current_provicer = ProviderServiceInfo.get_singleton_instance()
            get_frameinfo = CallerFrameServiceInfo()

            self._services[Symbols.provider] = get_current_provicer
            self._services[Symbols.provider_root] = ValueServiceInfo(self)
            self._services[Symbols.provider_parent] = GetAttrServiceInfo('_parent')
            self._services[Symbols.cache] = GetAttrServiceInfo('_scoped_cache')
            self._services[Symbols.missing_resolver] = ValueServiceInfo(ServiceInfoChainResolver())
            self._services[Symbols.caller_frame] = get_frameinfo

            self.__init_hooks = []
            self.__init_exc = None

            # service alias
            for name in SERVICEPROVIDER_NAMING_CONVENTION:
                self._services[name] = get_current_provicer
            self._services[ServiceProvider] = get_current_provicer
            self._services[IServiceProvider] = get_current_provicer
            self._services[inspect.FrameInfo] = get_frameinfo

            # options
            self._services[Symbols.provider_options] = ValueServiceInfo(MappingProxyType(
                dict(
                    auto_enter=auto_enter
                )
            ))

        assert self._root is not None

    def add_init_hook(self, func: Callable) -> None:
        func = wrap_signature(func)
        if self.__init_hooks is not None:
            with self._lock:
                if self.__init_hooks is not None:
                    self.__init_hooks.append(func)
                    return
        raise RuntimeError('Cannot add init hook after initialized.')

    def __ensure_init_hooks_called(self) -> None:
        if self.__init_hooks is not None or self.__init_exc is not None:
            with self._lock:
                if self.__init_exc is not None:
                    raise self.__init_exc
                if self.__init_hooks is not None:
                    _logger.debug('call init hooks')
                    hooks = self.__init_hooks
                    self.__init_hooks = None

                    disposable = self._services.add(Symbols.at_init, ValueServiceInfo(True))
                    try:
                        for func in hooks:
                            func(self)
                    except Exception as e:
                        self.__init_exc = e
                        raise
                    disposable()
                    self._services.add(Symbols.at_init, ValueServiceInfo(False))

    def _get_service_info(self, key: Hashable) -> IServiceInfo:
        try:
            return self._services[key]
        except KeyError:
            pass
        # load missing resolver and resolve service info.
        resolver = self._services[Symbols.missing_resolver].get_service(self)
        return resolver.get(self, key)

    @overload
    def __getitem__[T](self, key: NamedType[T]) -> T: ...
    @overload
    def __getitem__[T](self, key: TypedSymbol[T]) -> T: ...
    @overload
    def __getitem__(self, key: Hashable) -> object: ...
    @override
    def __getitem__(self, key: Hashable) -> object: # type: ignore
        _logger.debug('get service by key: %r', key)
        self._root.__ensure_init_hooks_called()
        service_info = self._get_service_info(key)
        try:
            return service_info.get_service(self)
        except ServiceNotFoundError as err:
            raise ServiceNotFoundError(key, *err.resolve_chain)

    @overload
    def get[T, TD](self, key: NamedType[T], d: TD=None) -> T | TD: ...
    @overload
    def get[T, TD](self, key: TypedSymbol[T], d: TD=None) -> T | TD: ...
    @overload
    def get(self, key: Hashable, d: object=None) -> object: ...
    @override
    def get(self, key: Hashable, d: object=None) -> object: # type: ignore
        '''
        Get a service by key with default value.
        '''
        try:
            return self[key]
        except ServiceNotFoundError as err:
            if len(err.resolve_chain) == 1:
                return d
            raise

    @overload
    def get_many[T](self, key: NamedType[T]) -> list[T]: ...
    @overload
    def get_many[T](self, key: TypedSymbol[T]) -> list[T]: ...
    @overload
    def get_many(self, key: Hashable) -> list[Any]: ...
    @override
    def get_many(self, key: Hashable) -> list[Any]: # type: ignore
        '''
        Get services by key.

        ### example

        when you registered multi services with the same key,
        you can get them all:

        ``` py
        provider.register_value('a', 1)
        provider.register_value('a', 2)
        assert provider.get_many('a') == [2, 1] # rev order
        ```
        '''
        _logger.debug('Get services by key: %r', key)
        self._root.__ensure_init_hooks_called()
        service_infos: Iterable[IServiceInfo] = self._services.get_many(key)
        try:
            return [si.get_service(self) for si in service_infos]
        except ServiceNotFoundError as err:
            raise ServiceNotFoundError(key, *err.resolve_chain)

    def resolve[R](self, factory: Callable[..., R], *,
            follow: bool = False,
            kwargs: Mapping[str, Any] | None = None
        ) -> R:
        '''
        Resolve the factory direct without register.
        '''
        return wrap_signature(factory, follow=follow, override_kwargs=kwargs)(self)

    @override
    def enter[T](self, context: AllSupportsContext[T]) -> T:
        with self._lock:
            if self._exit_stack is None:
                self._exit_stack = ExitStack()
            return self._exit_stack.enter_context(context)

    def __enter__(self) -> Self:
        return self

    def __exit__(self,
            exc_type: type | None,
            exc_val: BaseException | None,
            exc_tb: types.TracebackType | None, /
        ) -> None:
        with self._lock:
            if self._exit_stack is not None:
                self._exit_stack.__exit__(exc_type, exc_val, exc_tb)
                self._exit_stack = None
        _logger.debug('%r is exited.', self)

    def freeze_key(self, key: Hashable) -> None:
        '''
        Freeze key to avoid registering it later.

        Only effective for the current ServiceProvider.
        '''
        self._services.freeze_key(key)

    def register_service_info(self, key: Hashable, service_info: IServiceInfo) -> Disposable:
        '''
        register a `IServiceInfo` by key.
        '''
        if not isinstance(service_info, IServiceInfo):
            raise TypeError('service_info must be instance of IServiceInfo.')
        _logger.debug('register %r with key %r', service_info, key)

        disposable = self._services.add(key, service_info)
        match key:
            case NamedType(type=k):
                disposable += self._services.add(_NamedTypeListKey(k), ValueServiceInfo(key))

        return disposable

    @overload
    def register[T](self, key: NamedType[T], factory: Type[T], lifetime: LifeTime) -> Disposable: ...
    @overload
    def register(self, key: Hashable, factory: Callable[..., Any], lifetime: LifeTime) -> Disposable: ...
    def register(self, key: Hashable, factory: Callable[..., Any], lifetime: LifeTime) -> Disposable:
        '''
        register a service factory by key.

        `factory` accept a function which require one or zero parameter.
        if the count of parameter is 1, pass a `IServiceProvider` as the argument.
        '''
        return self.register_service_info(key, create_lifetime_service_info(self, key, factory, lifetime))

    @overload
    def register_singleton[T](self, key: NamedType[T], factory: Type[T]) -> Disposable: ...
    @overload
    def register_singleton(self, key: Hashable, factory: Callable[..., Any]) -> Disposable: ...
    def register_singleton(self, key: Hashable, factory: Callable[..., Any]) -> Disposable:
        '''
        register a service factory by key.

        `factory` accept a function which require one or zero parameter.
        if the count of parameter is 1, pass a `IServiceProvider` as the argument.
        '''
        return self.register(key, factory, LifeTime.singleton)

    @overload
    def register_scoped[T](self, key: NamedType[T], factory: Type[T]) -> Disposable: ...
    @overload
    def register_scoped(self, key: Hashable, factory: Callable[..., Any]) -> Disposable: ...
    def register_scoped(self, key: Hashable, factory: Callable[..., Any]) -> Disposable:
        '''
        register a service factory by key.

        `factory` accept a function which require one or zero parameter.
        if the count of parameter is 1, pass a `IServiceProvider` as the argument.
        '''
        return self.register(key, factory, LifeTime.scoped)

    @overload
    def register_transient[T](self, key: NamedType[T], factory: Type[T]) -> Disposable: ...
    @overload
    def register_transient(self, key: Hashable, factory: Callable[..., Any]) -> Disposable: ...
    def register_transient(self, key: Hashable, factory: Callable[..., Any]) -> Disposable:
        '''
        register a service factory by key.

        `factory` accept a function which require one or zero parameter.
        if the count of parameter is 1, pass a `IServiceProvider` as the argument.
        '''
        return self.register(key, factory, LifeTime.transient)

    @overload
    def register_value[T](self, key: NamedType[T], value: T) -> Disposable: ...
    @overload
    def register_value(self, key: Hashable, value: object) -> Disposable: ...
    def register_value(self, key: Hashable, value: object) -> Disposable:
        '''
        register a value by key.

        equals `register_transient(key, lambda ioc: value)`
        '''
        return self.register_service_info(key, ValueServiceInfo(value))

    def register_group(self, key: Hashable, keys: Iterable[Hashable]) -> Disposable:
        '''
        Register a group `key` for get other `keys`.

        For example:

        ```
        provider.register_value('str', 'name')
        provider.register_value('int', 1)
        provider.register_group('any', ['str', 'int'])
        assert provider['any'] == ('name', 1)
        ```

        Is equals `register_transient(key, lambda ioc: tuple(ioc[k] for k in keys))`
        '''
        return self.register_service_info(key, GetGroupServiceInfo(keys))

    def register_bind(self, new_key: Hashable, target_key: Hashable) -> Disposable:
        '''
        bind `new_key` to `target_key` so
        you can use `new_key` as key to get value from service provider.

        equals `register_transient(new_key, lambda ioc: ioc[target_key])`
        '''
        return self.register_service_info(new_key, BindedServiceInfo(target_key))

    @override
    def scope(self, *, use_lock: bool=False) -> 'ServiceProvider':
        '''
        Create a scoped service provider for get scoped services.

        By default, scoped IServiceProvider is not thread safely,
        set `use_lock` to `True` can change this.
        '''
        ssp = ServiceProvider(
            _services=self._services.scope(use_lock=use_lock),
            _parent=self,
            _use_lock=use_lock,
        )
        return self.enter(ssp)


__all__ = ['ServiceProvider']



# -*- coding: utf-8 -*-
# 
# Copyright (c) 2025~2999 - Cologler <skyoflw@gmail.com>
# ----------
# 
# ----------

import types
from collections.abc import Hashable, Iterator, MutableMapping
from contextlib import AbstractContextManager, nullcontext
from threading import RLock
from typing import Callable, TypedDict

from typing_extensions import ReadOnly

_NULL_CONTEXT = nullcontext()

class ProviderOptions(TypedDict):
    auto_enter: ReadOnly[bool]


class LockedMapping[TK: Hashable, TV](MutableMapping[TK, TV]):
    def __init__(self, use_lock: bool) -> None:
        super().__init__()
        self._dict: dict[TK, TV] = dict()
        # The lock may be acquired multiple times,
        # as it is used to prevent repeated calls.
        self._lock = RLock() if use_lock else _NULL_CONTEXT

    @property
    def lock(self) -> AbstractContextManager:
        return self._lock

    def __iter__(self) -> Iterator[TK]:
        with self._lock:
            return iter(self._dict)

    def __len__(self) -> int:
        with self._lock:
            return len(self._dict)

    def __getitem__(self, key: TK) -> TV:
        with self._lock:
            return self._dict[key]

    def __setitem__(self, key: TK, value: TV) -> None:
        with self._lock:
            self._dict[key] = value

    def __delitem__(self, key: TK) -> None:
        with self._lock:
            del self._dict[key]


class Disposable:
    __slots__ = ('dispose',)

    def __init__(self, dispose: Callable[[], None]) -> None:
        self.dispose = dispose

    def __call__(self) -> None:
        if dispose := self.dispose:
            self.dispose = None
            dispose()

    def __enter__(self) -> 'Disposable':
        return self

    def __exit__(self,
            exc_type: type | None,
            exc_val: BaseException | None,
            exc_tb: types.TracebackType | None, /
        ) -> None:
        if dispose := self.dispose:
            self.dispose = None
            dispose()

    def __add__(self, other: 'Disposable') -> 'Disposable':
        if isinstance(other, Disposable):
            def dispose() -> None:
                self()
                other()
            return Disposable(dispose)
        return NotImplemented



# -*- coding: utf-8 -*-
# 
# Copyright (c) 2025~2999 - Cologler <skyoflw@gmail.com>
# ----------
#
# ----------

import types
from typing import ForwardRef, Type, get_args

# symbol is primitive type, import any modules from . is not allowed.


class _Symbol:
    '''
    Symbol with description.
    '''

    __slots__ = ('_name', )

    def __init__(self, name: str='') -> None:
        self._name = name

    def __str__(self) -> str:
        return f'Symbol({self._name})'

    def __repr__(self) -> str:
        return f'Symbol({self._name!r})'


class TypedSymbol[T](_Symbol):
    '''
    Symbol with type.

    MUST use `TypedSymbol[...](...)` instead of `TypedSymbol(...)` directly.
    '''

    __slots__ = (
        '__orig_class__',
        '_type', # for cached property
    )

    def __str__(self) -> str:
        ta = self._get_type_args()
        tn = self._get_type_args_repr(ta)
        return f'TypedSymbol[{tn}]({self._name})'

    def __repr__(self) -> str:
        ta = self._get_type_args()
        tn = self._get_type_args_repr(ta)
        return f'TypedSymbol[{tn}]({self._name!r})'

    def _get_type_args(self) -> Type[T]:
        if (oc := getattr(self, '__orig_class__', None)) is not None:
            return get_args(oc)[0]
        raise TypeError('TypedSymbol is created without type args')

    @classmethod
    def _get_type_args_repr(cls, o: object) -> str:
        try:
            if isinstance(o, ForwardRef):
                return o.__forward_arg__
            elif isinstance(o, types.UnionType):
                return ' | '.join(cls._get_type_args_repr(x) for x in get_args(o))
            else:
                return getattr(o, '__name__', str(o))
        except AttributeError:
            return str(o)

    def get_type(self) -> type:
        '''
        Get the type of this symbol
        '''
        if not hasattr(self, '_type'):
            self._type = self._get_type_args()
        return self._type



# -*- coding: utf-8 -*-
# 
# Copyright (c) 2025~2999 - Cologler <skyoflw@gmail.com>
# ----------
# 
# ----------

import types
from abc import ABC, abstractmethod
from collections.abc import Callable, Hashable
from contextlib import AbstractContextManager
from enum import Enum
from typing import Any, Protocol, Self, overload, runtime_checkable

from ._primitive_symbol import TypedSymbol
from .keys import NamedType


class LifeTime(Enum):
    '''
    Never cache.
    '''
    transient = 0

    '''
    Value is cached per IServiceProvider scope.
    '''
    scoped = 1

    '''
    Value is cached on the IServiceInfo,
    and constructed using the IServiceProvider that owns this IServiceInfo.
    '''
    singleton = 2


@runtime_checkable
class SupportsContext[T](Protocol):
    def __enter__(self) -> T: ...
    def __exit__(self,
            exc_type: type | None,
            exc_val: BaseException | None,
            exc_tb: types.TracebackType | None, /
        ) -> None: ...

type AllSupportsContext[T] = SupportsContext[T] | AbstractContextManager[T]


class IServiceInfo[T](ABC):
    __slots__ = ()

    @abstractmethod
    def get_service(self, provider: 'IServiceProvider', /) -> T:
        raise NotImplementedError


class IServiceProvider:
    '''
    the base interface for `ServiceProvider`.
    '''
    def __enter__(self) -> Self: ...
    def __exit__(self,
            exc_type: type | None,
            exc_val: BaseException | None,
            exc_tb: types.TracebackType | None, /
        ) -> None: ...

    @overload
    def __getitem__[T](self, key: NamedType[T]) -> T: ...
    @overload
    def __getitem__[T](self, key: TypedSymbol[T]) -> T: ...
    @overload
    def __getitem__(self, key: Hashable) -> object: ...
    @abstractmethod
    def __getitem__(self, key: Hashable) -> object:
        '''
        Get a service by key.
        '''
        raise NotImplementedError

    @overload
    def get[T, TD](self, key: NamedType[T], d: TD=None) -> T | TD: ...
    @overload
    def get[T, TD](self, key: TypedSymbol[T], d: TD=None) -> T | TD: ...
    @overload
    def get(self, key: Hashable, d: object=None) -> object: ...
    @abstractmethod
    def get(self, key: Hashable, d: object=None) -> object:
        '''
        Get a service by key with default value.
        '''
        raise NotImplementedError

    @overload
    def get_many[T](self, key: NamedType[T]) -> list[T]: ...
    @overload
    def get_many[T](self, key: TypedSymbol[T]) -> list[T]: ...
    @overload
    def get_many(self, key: Hashable) -> list[Any]: ...
    @abstractmethod
    def get_many(self, key: Hashable) -> list[Any]:
        '''
        Get services by key.
        '''
        raise NotImplementedError

    @abstractmethod
    def resolve[R](self, factory: Callable[..., R]) -> R:
        '''
        Resolve the factory direct without register.
        '''
        raise NotImplementedError

    @abstractmethod
    def scope(self, *, use_lock: bool=False) -> 'IServiceProvider':
        '''
        Create a scoped service provider for get scoped services.

        By default, scoped IServiceProvider is not thread safely,
        set `use_lock` to `True` can change this.
        '''
        raise NotImplementedError

    @abstractmethod
    def enter[T](self, context: AllSupportsContext[T]) -> T:
        '''
        Enter the context, so that this context exits together when the current provider exits.

        Returns the result of the `context.__enter__()` method.
        '''
        raise NotImplementedError


class Factory[T](Protocol):
    __slots__ = ()
    def __call__(self, provider: IServiceProvider, /) -> T: ...



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018~2999 - Cologler <skyoflw@gmail.com>
# ----------
#
# ----------

from ._bases import IServiceProvider, LifeTime
from .annotations import DontInject, InjectBy, InjectByGroup, InjectFrom, InjectWithValue
from .err import ServiceNotFoundError
from .ioc import ServiceProvider

__all__ = [
    'DontInject',
    'InjectBy',
    'InjectByGroup',
    'InjectFrom',
    'InjectWithValue',
    'IServiceProvider',
    'LifeTime',
    'ServiceNotFoundError',
    'ServiceProvider',
]



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018~2999 - Cologler <skyoflw@gmail.com>
# ----------
#
# ----------

import sys
import types
from collections.abc import Hashable
from contextlib import nullcontext
from threading import RLock
from typing import Any, get_args, get_origin, override

from ._bases import IServiceInfo, IServiceProvider
from ._service_info import FactoryServiceInfo, GetManyServiceInfo, ValueServiceInfo
from ._utils import wrap_signature
from .err import ServiceNotFoundError


class IServiceInfoResolver:
    '''
    the base class for dynamic resolve `IServiceInfo`.
    '''

    def get(self, provider: IServiceProvider, key: Hashable, /) -> IServiceInfo[Any]:
        '''
        Get the `IServiceInfo` from resolver.
        '''
        raise ServiceNotFoundError(key)

    def __add__(self, other: 'IServiceInfoResolver') -> 'IServiceInfoResolver':
        new_resolver = ServiceInfoChainResolver()
        new_resolver.chain.append(self)
        new_resolver.append(other)
        return new_resolver

    def cache(self, *, sync: bool=False) -> 'IServiceInfoResolver':
        '''
        Returns a `IServiceInfoResolver` to cache all `IServiceInfo`s from this `IServiceInfoResolver`.

        All values won't dynamic update after the first resolved.
        '''
        return CacheServiceInfoResolver(self, sync=sync)


class ServiceInfoChainResolver(IServiceInfoResolver):
    '''
    A chained resolver for resolve IServiceInfos from each `IServiceInfoResolver`
    '''

    def __init__(self, *resolvers: IServiceInfoResolver) -> None:
        self.chain = list(resolvers)

    def get(self, provider: IServiceProvider, key: Hashable) -> IServiceInfo[Any]:
        for resolver in self.chain:
            try:
                return resolver.get(provider, key)
            except ServiceNotFoundError:
                pass
        return super().get(provider, key)

    def append(self, other: IServiceInfoResolver) -> None:
        if isinstance(other, ServiceInfoChainResolver):
            self.chain.extend(other.chain)
        else:
            self.chain.append(other)

    def __add__(self, other: 'IServiceInfoResolver') -> 'ServiceInfoChainResolver':
        new_resolver = ServiceInfoChainResolver()
        new_resolver.chain.extend(self.chain)
        new_resolver.append(other)
        return new_resolver


class CacheServiceInfoResolver(IServiceInfoResolver):
    '''
    a helper resolver for cache values from other `IServiceInfoResolver`

    NOTE:
    if a `IServiceInfo` is affect by `provider`, you should not cache it.
    `CacheServiceInfoResolver` only cache by the `key` and ignore the `provider` arguments.
    '''

    def __init__(self, base_resolver: IServiceInfoResolver, *, sync: bool=False) -> None:
        super().__init__()
        self._base_resolver = base_resolver
        self._cache = {}
        self._lock = RLock() if sync else nullcontext()

    def get(self, provider: IServiceProvider, key: Hashable) -> IServiceInfo[Any]:
        try:
            return self._cache[key]
        except KeyError:
            pass
        with self._lock:
            try:
                return self._cache[key]
            except KeyError:
                pass
            service_info = self._base_resolver.get(provider, key)
            self._cache[key] = service_info
            return service_info

    def cache(self, *, sync: bool=False) -> 'IServiceInfoResolver':
        if sync and isinstance(self._lock, nullcontext):
            return CacheServiceInfoResolver(self, sync=sync)
        return self


class ImportServiceInfoResolver(IServiceInfoResolver):
    '''
    Dynamic resolve `IServiceInfo` if the key is a module name.

    - Relative import is not allowed;
    - If the key is `{module_name}`, only lookup modules from `sys.modules`;
    - If the key is `module::{module_name}` (startswith `module::`), the resolver will try to import it;
    '''

    @override
    def get(self, provider: IServiceProvider, key: Hashable, /) -> IServiceInfo[types.ModuleType]:
        if isinstance(key, str) and not key.startswith('.'): # relative import is not allows
            if key.startswith('module::'):
                module_name = key.removeprefix('module::')
                import importlib
                try:
                    module = importlib.import_module(module_name)
                    return ValueServiceInfo(module)
                except (TypeError, ModuleNotFoundError):
                    pass
            else:
                module_name = key
                if module := sys.modules.get(module_name):
                    return ValueServiceInfo(module)
        return super().get(provider, key)


class TypesServiceInfoResolver(IServiceInfoResolver):
    '''
    Dynamic resolve `IServiceInfo` if the key is a type instance.
    '''

    def get(self, provider: IServiceProvider, key: Hashable) -> IServiceInfo[Any]:
        if isinstance(key, type):
            return FactoryServiceInfo(wrap_signature(key))
        return super().get(provider, key)


class TypeNameServiceInfoResolver(IServiceInfoResolver):
    '''
    dynamic resolve `IServiceInfo` if the key is a type name or qualname.
    '''

    def _get_type(self, key: Hashable) -> type | None:
        if isinstance(key, str):
            for klass in object.__subclasses__():
                if getattr(klass, '__name__', None) == key:
                    return klass
                if getattr(klass, '__qualname__', None) == key:
                    return klass
        # None

    def get(self, provider: IServiceProvider, key: Hashable) -> IServiceInfo[Any]:
        klass = self._get_type(str)
        if klass is not None:
            return FactoryServiceInfo(wrap_signature(klass))
        return super().get(provider, key)


class GenericListAsGetManyServiceInfoResolver(IServiceInfoResolver):
    '''
    Dynamic resolve `IServiceInfo` for `list[T]` to `.get_many(T)`
    '''
    def __init__(self,
            for_list: bool=True,
            for_tuple: bool=False,
        ) -> None:
        super().__init__()
        self._for_list = for_list
        self._for_tuple = for_tuple

    @override
    def get(self, provider: IServiceProvider, key: Hashable) -> IServiceInfo[Any]:
        if isinstance(key, types.GenericAlias) and get_origin(key) is list:
            return GetManyServiceInfo(get_args(key)[0])
        return super().get(provider, key)
